import { ApolloQueryResult } from "@apollo/client";
import { IApolloClient } from "@deep-foundation/hasura/client";
import { Link, MinilinksResult } from "./minilinks";
export declare const ALLOWED_IDS: number[];
export declare const DENIED_IDS: number[];
export declare const GLOBAL_ID_PACKAGE = 2;
export declare const GLOBAL_ID_CONTAIN = 3;
export declare const GLOBAL_ID_STRING = 5;
export declare const GLOBAL_ID_NUMBER = 6;
export declare const GLOBAL_ID_OBJECT = 7;
export declare const GLOBAL_ID_ANY = 8;
export declare const GLOBAL_ID_PROMISE = 9;
export declare const GLOBAL_ID_THEN = 10;
export declare const GLOBAL_ID_RESOLVED = 11;
export declare const GLOBAL_ID_REJECTED = 12;
export declare const GLOBAL_ID_TREE = 36;
export declare const GLOBAL_ID_INCLUDE_DOWN = 37;
export declare const GLOBAL_ID_INCLUDE_UP = 38;
export declare const GLOBAL_ID_INCLUDE_NODE = 39;
export declare const GLOBAL_ID_CONTAIN_TREE = 40;
export declare const GLOBAL_ID_PACKAGE_NAMESPACE = 43;
export declare const GLOBAL_ID_PACKAGE_ACTIVE = 45;
export declare const GLOBAL_ID_PACKAGE_VERSION = 46;
export declare const GLOBAL_ID_HANDLE_UPDATE = 50;
export declare const GLOBAL_ID_JOIN = 66;
export declare const GLOBAL_ID_ALLOW_ADMIN = 71;
export declare const GLOBAL_ID_SELECTOR_FILTER = 75;
export declare const GLOBAL_ID_INCLUDE_IN = 113;
export declare const GLOBAL_ID_INCLUDE_OUT = 114;
export declare const GLOBAL_ID_INCLUDE_FROM_CURRENT = 115;
export declare const GLOBAL_ID_INCLUDE_TO_CURRENT = 116;
export declare const GLOBAL_ID_INCLUDE_CURRENT_FROM = 117;
export declare const GLOBAL_ID_INCLUDE_CURRENT_TO = 118;
export declare const GLOBAL_ID_INCLUDE_FROM_CURRENT_TO = 119;
export declare const GLOBAL_ID_INCLUDE_TO_CURRENT_FROM = 120;
export declare function parseJwt(token: any): {
    userId: number;
    role: string;
    roles: string[];
    [key: string]: any;
};
export interface DeepClientOptions<L = Link<number>> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep?: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    silent?: boolean;
}
export interface DeepClientResult<R> extends ApolloQueryResult<R> {
    error?: any;
}
export declare type DeepClientPackageSelector = string;
export declare type DeepClientPackageContain = string;
export declare type DeepClientLinkId = number;
export declare type DeepClientStartItem = DeepClientPackageSelector | DeepClientLinkId;
export declare type DeepClientPathItem = DeepClientPackageContain | boolean;
export interface DeepClientInstance<L = Link<number>> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    stringify(any?: any): string;
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
    guest(options: DeepClientGuestOptions): Promise<DeepClientAuthResult>;
    jwt(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    login(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    logout(): Promise<DeepClientAuthResult>;
    can(objectIds: number[], subjectIds: number[], actionIds: number[]): Promise<boolean>;
}
export interface DeepClientAuthResult {
    linkId?: number;
    token?: string;
    error?: any;
}
export interface DeepClientGuestOptions {
    relogin?: boolean;
}
export interface DeepClientJWTOptions {
    linkId?: number;
    token?: string;
    relogin?: boolean;
}
export declare class DeepClient<L = Link<number>> implements DeepClientInstance<L> {
    linkId?: number;
    token?: string;
    handleAuth?: (linkId?: number, token?: string) => any;
    deep: DeepClientInstance<L>;
    apolloClient?: IApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    silent: boolean;
    _silent(options?: Partial<{
        silent?: boolean;
    }>): boolean;
    constructor(options: DeepClientOptions<L>);
    stringify(any?: any): string;
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects?: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
        silent?: boolean;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number, options?: {
        results: boolean;
    }): Promise<any>;
    _serialize: {
        link: {
            value: string;
            relations: {
                string: string;
                number: string;
                object: string;
                to: string;
                from: string;
                in: string;
                out: string;
                type: string;
                typed: string;
            };
        };
        value: {
            relations: {
                link: string;
            };
        };
    };
    _boolExpFields: {
        _and: boolean;
        _not: boolean;
        _or: boolean;
    };
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    pathToWhere(start: DeepClientStartItem, ...path: DeepClientPathItem[]): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
    guest(options?: DeepClientGuestOptions): Promise<DeepClientAuthResult>;
    jwt(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    whoami(): Promise<number | undefined>;
    login(options: DeepClientJWTOptions): Promise<DeepClientAuthResult>;
    logout(): Promise<DeepClientAuthResult>;
    can(objectIds: number | number[], subjectIds: number | number[], actionIds: number | number[], userIds?: number | number[]): Promise<boolean>;
}
export declare const JWT: import("@apollo/client").DocumentNode;
export declare const WHOISME: import("@apollo/client").DocumentNode;
export declare const GUEST: import("@apollo/client").DocumentNode;
export declare function useAuthNode(): [number, (value: number) => any, () => any];
export declare function useDeep(apolloClientProps?: IApolloClient<any>): DeepClient<Link<number>>;
export declare function useDeepQuery(query: any, options?: any): any;
