"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.packagerPublishCore = exports.packagerInstallCore = exports.typeDefs = exports.typeDefsString = void 0;
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const gists_1 = __importDefault(require("gists"));
const client_1 = require("@deep-foundation/hasura/client");
const packager_1 = require("../packager");
const apollo_server_express_1 = require("apollo-server-express");
const client_2 = require("../client");
const os_1 = __importDefault(require("os"));
const apollo_server_core_1 = require("apollo-server-core");
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const lodash_1 = require("lodash");
const tmpdir = os_1.default.tmpdir();
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({ apolloClient: client });
const packager = new packager_1.Packager(deep);
exports.typeDefsString = `
  type Query {
    packager_install(input: PackagerInstallInput): PackagerInstallOutput
    packager_publish(input: PackagerPublishInput): PackagerPublishOutput
  }
  input PackagerInstallInput {
    address: String
  }
  type PackagerInstallOutput {
    ids: [Int]
    errors: [String]
    packageId: Int
  }
  input PackagerPublishInput {
    id: Int
    address: String
  }
  type PackagerPublishOutput {
    errors: [String]
    address: String
  }
`;
exports.typeDefs = (0, graphql_tag_1.default) `${exports.typeDefsString}`;
const packagerInstallCore = (errors = [], address) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const uri = new url_1.default.URL(address);
    const isGist = uri.hostname === 'gist.github.com';
    const isNpm = uri.hostname === 'npmjs.com' || uri.hostname === 'www.npmjs.com';
    if (isGist) {
        const username = uri.username;
        const gistId = uri.pathname.split('/')[2];
        if (!username) {
            errors.push('gist requires token');
        }
        if (!gistId) {
            errors.push('gist valid address like http://gist.github.com/account/gist');
        }
        if (errors.length)
            return { errors };
        const gists = new gists_1.default({ token: username });
        const result = yield gists.get(gistId);
        const files = (_a = result === null || result === void 0 ? void 0 : result.body) === null || _a === void 0 ? void 0 : _a.files;
        const deepPckgContent = (_b = files === null || files === void 0 ? void 0 : files['deep.json']) === null || _b === void 0 ? void 0 : _b.content;
        if (deepPckgContent) {
            try {
                const deepPckg = JSON.parse(deepPckgContent);
                const { ids, errors, packageId } = yield packager.import(deepPckg);
                return { ids, errors, packageId };
            }
            catch (e) {
                errors.push(String(e));
            }
        }
        else {
            errors.push(`deep.json not founded in gist`);
        }
    }
    else {
        errors.push(`address is not valid`);
    }
});
exports.packagerInstallCore = packagerInstallCore;
const packagerPublishCore = (errors = [], address, id) => __awaiter(void 0, void 0, void 0, function* () {
    var _c, _d, _e, _f, _g, _h;
    if (!id)
        return { errors: ['!id'] };
    const packageResults = yield deep.select(id);
    if (!((_c = packageResults.data) === null || _c === void 0 ? void 0 : _c.length))
        return { errors: ['!package'] };
    const uri = new url_1.default.URL(address);
    const isGist = uri.hostname === 'gist.github.com';
    const isNpm = uri.hostname === 'npmjs.com' || uri.hostname === 'www.npmjs.com';
    if (isGist) {
        const username = uri.username;
        const gistId = uri.pathname.split('/')[2];
        if (!username) {
            errors.push('gist requires token');
        }
        if (errors.length)
            return { errors };
        if (gistId) {
            const deepPckgContent = yield packager.export({ packageLinkId: id });
            if ((_d = deepPckgContent === null || deepPckgContent === void 0 ? void 0 : deepPckgContent.errors) === null || _d === void 0 ? void 0 : _d.length) {
                errors.push(...deepPckgContent.errors);
            }
            if ((0, lodash_1.isEmpty)(deepPckgContent === null || deepPckgContent === void 0 ? void 0 : deepPckgContent.package)) {
                errors.push('!package');
            }
            if ((0, lodash_1.isEmpty)(deepPckgContent === null || deepPckgContent === void 0 ? void 0 : deepPckgContent.data)) {
                errors.push('!data');
            }
            if (errors.length)
                return { errors };
            const gists = new gists_1.default({ token: username });
            const result = yield gists.edit(gistId, { files: { 'deep.json': { content: JSON.stringify(deepPckgContent) } } });
            if ((_e = result === null || result === void 0 ? void 0 : result.body) === null || _e === void 0 ? void 0 : _e.id)
                return { errors, address: `https://${username}@gist.github.com${uri.pathname}` };
        }
        else {
            const deepPckgContent = yield packager.export({ packageLinkId: id });
            if ((_f = deepPckgContent === null || deepPckgContent === void 0 ? void 0 : deepPckgContent.errors) === null || _f === void 0 ? void 0 : _f.length) {
                errors.push(...deepPckgContent.errors);
                return { errors };
            }
            const gists = new gists_1.default({ token: username });
            const result = yield gists.create({ files: { 'deep.json': { content: JSON.stringify(deepPckgContent) } } });
            if ((_g = result === null || result === void 0 ? void 0 : result.body) === null || _g === void 0 ? void 0 : _g.id)
                return { errors, address: `https://${username}@gist.github.com${path_1.default.normalize(`${uri.pathname}/${(_h = result === null || result === void 0 ? void 0 : result.body) === null || _h === void 0 ? void 0 : _h.id}`)}` };
        }
        if (errors.length)
            return { errors };
    }
});
exports.packagerPublishCore = packagerPublishCore;
const resolvers = {
    Query: {
        packager_install: (source, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
            var _j, _k, _l;
            const errors = [];
            if (((_j = context === null || context === void 0 ? void 0 : context.headers) === null || _j === void 0 ? void 0 : _j['x-hasura-role']) !== 'admin' &&
                !(yield deep.can(yield deep.id('@deep-foundation/core', 'AllowPackagerInstall'), +((_k = context === null || context === void 0 ? void 0 : context.headers) === null || _k === void 0 ? void 0 : _k['x-hasura-user-id']), yield deep.id('@deep-foundation/core', 'AllowPackagerInstall')))) {
                errors.push('cant');
                return { errors };
            }
            const { address } = args.input;
            const result = yield (0, exports.packagerInstallCore)(errors, (_l = args.input) === null || _l === void 0 ? void 0 : _l.address);
            if (result)
                return result;
            return { ids: [], errors };
        }),
        packager_publish: (source, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
            var _m, _o;
            const errors = [];
            if (((_m = context === null || context === void 0 ? void 0 : context.headers) === null || _m === void 0 ? void 0 : _m['x-hasura-role']) !== 'admin' &&
                !(yield deep.can(yield deep.id('@deep-foundation/core', 'AllowPackagerPublish'), +((_o = context === null || context === void 0 ? void 0 : context.headers) === null || _o === void 0 ? void 0 : _o['x-hasura-user-id']), yield deep.id('@deep-foundation/core', 'AllowPackagerPublish')))) {
                errors.push('cant');
                return { errors };
            }
            const { id, address } = args.input;
            const result = yield (0, exports.packagerPublishCore)(errors, address, id);
            if (result)
                return result;
            return { errors };
        }),
    }
};
const context = ({ req }) => {
    return { headers: req.headers };
};
const generateApolloServer = (httpServer) => {
    return new apollo_server_express_1.ApolloServer({
        introspection: true,
        typeDefs: exports.typeDefs,
        resolvers,
        context,
        plugins: [
            (0, apollo_server_core_1.ApolloServerPluginDrainHttpServer)({ httpServer }),
            (0, apollo_server_core_1.ApolloServerPluginLandingPageGraphQLPlayground)()
        ]
    });
};
exports.default = generateApolloServer;
//# sourceMappingURL=packager.js.map