"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleScheduleMomemt = exports.insertPromise = exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const apollo_boost_1 = require("apollo-boost");
const promise_1 = require("../promise");
const client_2 = require("../client");
const links_1 = require("./links");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deeplinks:eh:scheduler');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
exports.api = new api_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
const insertPromise = (scheduleId) => __awaiter(void 0, void 0, void 0, function* () {
    const promiseTypeId = yield deep.id('@deep-foundation/core', 'Promise');
    const thenTypeId = yield deep.id('@deep-foundation/core', 'Then');
    const promise = (yield deep.insert({
        type_id: promiseTypeId,
    }, { name: 'IMPORT_HANDLER_JS_FILE' })).data[0];
    const then = (yield deep.insert({
        from_id: scheduleId,
        type_id: thenTypeId,
        to_id: promise === null || promise === void 0 ? void 0 : promise.id,
    }, { name: 'IMPORT_HANDLER' })).data[0];
    return {
        id: promise === null || promise === void 0 ? void 0 : promise.id,
        thenId: then === null || then === void 0 ? void 0 : then.id,
    };
});
exports.insertPromise = insertPromise;
function handleScheduleMomemt(moment) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    return __awaiter(this, void 0, void 0, function* () {
        const scheduleId = moment.payload.scheduleId;
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promise = yield (0, promise_1.findPromiseLink)({
            id: scheduleId, client: deep.apolloClient,
            Then: yield deep.id('@deep-foundation/core', 'Then'),
            Promise: yield deep.id('@deep-foundation/core', 'Promise'),
            Resolved: resolvedTypeId,
            Rejected: rejectedTypeId,
            Results: false,
        });
        log('promise: ', promise);
        if (promise) {
            const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
            const handleOperationTypeId = yield deep.id('@deep-foundation/core', 'HandleSchedule');
            const queryString = `query SELECT_CODE($scheduleId: bigint) { links(where: {
            type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
            in: {
              from_id: { _eq: ${yield deep.id('@deep-foundation/core', 'dockerSupportsJs')} },
              type_id: { _eq: ${handlerTypeId} },
              in: {
                from_id: { _eq: $scheduleId },
                type_id: { _eq: ${handleOperationTypeId} },
              }
            }
          }) {
            id
            value
            in(where: { type_id: { _eq: ${handlerTypeId} } }) {
              id
              in(where: { type_id: { _eq: ${handleOperationTypeId} } }) {
                id
              }
              support: from {
                id
                isolation: from {
                  id
                  value
                }
              }
            }
          } }`;
            const query = (0, apollo_boost_1.gql) `${queryString}`;
            const variables = {
                scheduleId
            };
            const handlersResult = yield client.query({ query, variables });
            const promises = [];
            const handleInsertsIds = [];
            const handlersWithCode = (_a = handlersResult === null || handlersResult === void 0 ? void 0 : handlersResult.data) === null || _a === void 0 ? void 0 : _a.links;
            log('handlersWithCode.length', handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
            if ((handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length) > 0) {
                log("handlersWithCode: ", JSON.stringify(handlersWithCode, null, 2));
                log(handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
                for (const handlerWithCode of handlersWithCode) {
                    const code = (_b = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.value) === null || _b === void 0 ? void 0 : _b.value;
                    const isolationValue = (_g = (_f = (_e = (_d = (_c = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.support) === null || _e === void 0 ? void 0 : _e.isolation) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                    const handlerId = (_j = (_h = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.id;
                    const handleInsertId = (_m = (_l = (_k = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _k === void 0 ? void 0 : _k[0]) === null || _l === void 0 ? void 0 : _l.in) === null || _m === void 0 ? void 0 : _m[0].id;
                    if (code && isolationValue && handlerId && handleInsertId) {
                        try {
                            promises.push(() => (0, links_1.useRunner)({ code, handlerId, handler: isolationValue, moment }));
                            handleInsertsIds.push(handleInsertId);
                        }
                        catch (e) {
                            error('error', e);
                        }
                    }
                    else {
                        promises.push(() => __awaiter(this, void 0, void 0, function* () { return Promise.reject(new Error('Code of a handler is not loaded.')); }));
                        handleInsertsIds.push(null);
                    }
                }
                const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
                const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
                yield (0, exports.insertPromise)(scheduleId);
                yield (0, links_1.processPromises)(promises, handleInsertsIds, promise.id, promiseResultTypeId, promiseReasonTypeId, resolvedTypeId, rejectedTypeId, log);
            }
        }
    });
}
exports.handleScheduleMomemt = handleScheduleMomemt;
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const event = req === null || req === void 0 ? void 0 : req.body;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        log(`event`, event);
        try {
            yield handleScheduleMomemt(event);
            return res.status(200).json({});
        }
        catch (e) {
            error('error', e);
            throw e;
        }
        return res.status(500).json({ error: 'notexplained' });
    }
    catch (e) {
        return res.status(500).json({ error: e.toString() });
    }
});
//# sourceMappingURL=scheduler.js.map